import csv as pcsv
import dataclasses
from typing import Optional, Tuple
import re
import datetime

import click


@dataclasses.dataclass
class Entry:
    date: str
    account: str
    memo: str
    location: str
    kind: str
    amount: float


def parse_dict(filename: str):
    words = {}
    with open(filename) as f:
        for line in f:
            for word in line.strip().split():
                words[word.lower()] = word
    for word in ('AG', 'TWINT', 'SBB'):
        words[word.lower()] = word
        
    return words


WORDS = parse_dict('/usr/share/dict/words')


def fix_case(line: str) -> str:
    words = []
    for i, word in enumerate(line.split()):
        word = WORDS.get(word.lower(), word.lower())
        if i == 0:
            word = word[0].upper() + word[1:]
        words.append(word)
    return ' '.join(words)


def parse_date(text: str) -> datetime.date:
    d, m, y = (int(x) for x in text.split('.'))
    return datetime.date(year=y, month=m, day=d)


def parse_memo(memo: str) -> Tuple[str, str, str]:
    location = ''
    kind = ''

    memo = memo.strip()

    match = re.match(r'^.+CARD NO. XX\S+\s+(.+)', memo)
    if match:
        memo = match.group(1)

    match = re.match(r'TWINT PURCHASE/SERVICE FROM \S+ FROM TELEPHONE NO. \S+\s+(.+)', memo)
    if match:
        memo = match.group(1)

    match = re.match(r'(DEBIT|CREDIT) CH\d+\s+(.+)', memo)
    if match:
        memo = match.group(2)

    memo = fix_case(memo)
    
    match = re.match(r'(.+) (\w+) (Switzerland|France)$', memo)
    if match:
        memo, city, country = match.groups()
        location = f'{city} {country}'

    return memo, location, kind


def parse(line: tuple) -> Optional[Entry]:
    # 13.07.2024;Entry;"TWINT PURCHASE/SERVICE FROM 12.07.2024 FROM TELEPHONE NO. +41792456887 MCDONALD'S ZÜRICH SIHLCITY ZUERICH";;-9.50;;
    if len(line) < 5 or not line[-1]:
        return None

    if not re.match(r'\d+\.\d+\.\d+', line[0]):
        return None

    date = parse_date(line[0])
    memo = line[2]
    memo, location, kind = parse_memo(memo)

    if line[3]:
        amount = float(line[3].replace("'", ""))
    else:
        amount = -float(line[4].replace("'", ""))

    if not kind:
        kind = line[-1]
        
    return Entry(date=str(date),
                 account="pf",
                 memo=memo,
                 location=location,
                 kind=kind,
                 amount=amount)


@click.command()
@click.option('--out', required=True, type=str)
@click.option('--csv', required=True, type=str)
def extract(out: str, csv: str):
    entries = []

    with open(csv) as f:
        dialect = pcsv.Sniffer().sniff(f.read(1024), ';')
        f.seek(0)
        for line in pcsv.reader(f, dialect=dialect):
            entry = parse(line)
            if entry:
                entries.append(entry)

    with open(out, 'w') as f:
        writer = pcsv.writer(f)
        for entry in entries:
            writer.writerow(dataclasses.astuple(entry))


if __name__ == '__main__':
    extract()
